#!/usr/bin/perl
#--- linux

#!/programmi/perl/bin/perl.exe
#--- win

# Author: PierLuigi Spinosa (pierluigi.spinosa@ittig.cnr.it)
# Copyright: ITTIG / CNR - Firenze
# Date: May 2011

use CGI;
use LWP::Simple;		# per download documento da rete
use IPC::Open2;		# per interscambio dati input/output con parser

# --- configurazione

$parserHome = "/usr/local/bin";
$htmlFile   = "/var/www/urn/nirvana3.html";
$logFile    = "/var/log/weblog/nirvana3.log";

# --- win
#$parserHome = '/programmi/nir/ParserRiferimenti2';
#$htmlFile   = '/programmi/apache group/apache2/htdocs/nirvana-win.html';
#$logFile    = '/programmi/apache group/apache2/logs/nirvana.log';

# --- acquisizione parametri da form

$cgi = new CGI;
$formato      = $cgi->param(formato);
$marcatura    = $cgi->param(marcatura);
$sepExt       = $cgi->param(sepExt);
$sepInt       = $cgi->param(sepInt);
$apertura     = $cgi->param(apertura);
$chiusura     = $cgi->param(chiusura);
$uscita       = $cgi->param(uscita);
$inputTipo    = $cgi->param(inputTipo);
$regione      = $cgi->param(regione);
$ente         = $cgi->param(ente);
$destinazione = $cgi->param(destinazione);
$resolver     = $cgi->param(resolver);
$base         = $cgi->param(base);
$baseadd      = $cgi->param(baseadd);

# --- gestione tipo di input (tastiera, disco, url) e lettura dati

if ($inputTipo eq 'tastiera') {
	$text = $cgi->param(inputTastiera) . "\n";
	$name = 'testo';
} 
elsif ($inputTipo eq 'upload') {
	$text = '';
	$fn = $cgi->upload('inputFile');
	while (<$fn>) {
		$text .= $_;
	}
	close $fn;
	$name = $cgi->param(inputFile);
}
elsif ($inputTipo eq 'rete') {
	$text = '';
	$url = $cgi->param(url);
	$text = get($url);
	$name = $url;
}

# --- gestione del nome del file

if ($name){
	$name =~ s/.*[\/\\]//;		# tolgo fino all'ultima /
	$name =~ s/\..*?$//;		# tolgo dall'ultimo punto alla fine
}

# --- gestione base address

if ($base eq 'none')
	{$baseadd = '';}
elsif ($inputTipo eq 'rete' and $base eq 'url')
	{$baseadd = $url;}
if ($baseadd and $baseadd !~ m!^http://!i)
	{$baseadd = "http://$baseadd";}

if ($resolver and $resolver !~ m!^http://!i)
	{$resolver = "http://$resolver";}


#print $cgi->header;
#print "<pre>\nformato=$formato\nmarcatura=$marcatura\napertura=$apertura\n" . 
	  "chiusura=$chiusura\nuscita=$uscita\ninputTipo=$inputTipo\ntext=$text\n" . 
	  "regione=$regione\nente=$ente\ndestinazione=$destinazione\nurl=$url\n" . 
	  "sepExt=$sepExt\nsepInt=$sepInt\nresolver=$resolver\n" . 
	  "base=$base\nbaseadd=$baseadd\n" . 
	  "parser=$parserHome\nhtmlFile=$htmlFile\nlogFile=$logFile\n</pre>\n";

# --- gestione errori

open (LOG, ">>$logFile") or $erroreLog="$!";

if (not $logFile or not $parserHome or not $htmlFile or $erroreLog or not $text) {
	print $cgi->header;
	print $cgi->start_html("Errore configurazione cgi");
	print $cgi->p("Errore: variabile del CGI parserHome non impostata.") if not $parserHome;
	print $cgi->p("Errore: variabile del CGI htmlFile non impostata.") if not $htmlFile;
	print $cgi->p("Errore: variabile del CGI logFile non impostata.") if not $logFile;
	print $cgi->p("Errore: apertura file di log: $erroreLog.") if $erroreLog;
	print $cgi->p("Errore: documento di input vuoto (da $inputTipo).") if not $text;
	print $cgi->end_html;
	exit;
}
elsif ($base eq 'user' and not $baseadd) {
	print $cgi->header;
	print $cgi->start_html("Errore configurazione cgi");
	print $cgi->p("Errore: variabile del CGI baseadd non impostata.");
	exit;
}

# --- preparazione chiamata parser

#--- linux
#$parser = "/usr/bin/perl -I$parserHome $parserHome\/URNparser.pl -i $formato -o $uscita -l keep ";

$parser = "$parserHome\/ParserRiferimenti3 -i $formato -o $uscita -l keep ";

# --- win
#$parser = "perl -I$parserHome $parserHome\/URNparser.pl -i $formato -o $uscita -l keep -v3";

#if ($marcatura eq 'none') {
#	$parser .= " -b \"$apertura\" -a \"$chiusura\" ";
#} else {

	$parser .= " -m $marcatura ";
#}

# --- impostazione estensione file

if ($uscita ne 'doc')
	{$ext = 'txt';}
elsif ($marcatura eq 'htmlnir')
	{$ext = 'html';}
elsif ($marcatura eq 'htmlris')
	{$ext = 'html';}
elsif ($marcatura eq 'dtdnir')
	{$ext = 'xml';}

if ($regione)
	{$parser .= " -R $regione ";}

if ($ente)
	{$parser .= " -E $ente ";}

if ($sepExt)
	{$parser .= " -S $sepExt ";}

if ($sepInt)
	{$parser .= " -s $sepInt ";}

if ($resolver)
	{$parser .= " -u \"$resolver\" ";}

# --- chiamata parser e apertura flussi input/output

#print $cgi->header;
#print "<pre>$parser</pre>\n";

$pid = open2(IN, OUT, "$parser");

print OUT $text;			# input per parser
close OUT;

while (<IN>) {				# output del parser
	$ris .= $_;
}
close IN;

#print $cgi->header;
#print "<pre>ris=\n$ris\n</pre>";
#exit;

# --- gestione file di log

#$urlPartenza=$cgi->referer();

$tempo = localtime;
($gs, $mm, $gg, $hms, $aa) = split(/ /,$tempo);
print LOG "$gg/$mm/$aa:$hms cli=" . ($cgi->remote_host()) . " lun=" . (length $text);

my @line;
if ($uscita ne 'doc'){ 					# --- calcolo n.o riferimenti trovati
	@line = split ("\n", $ris);
	print LOG " rif=$#line";
} else {
	$ris =~ /\<\!-- Inseriti n\.o (\d+) link ipetestuali/;
	print LOG " rif=$1";
}
print LOG " inp=$inputTipo for=$formato mar=$marcatura usc=$uscita des=$destinazione";
print LOG " reg=$regione\n ent=$ente ext=$sepExt int=$sepInt ris=$resolver";

close (LOG);

# --- gestione base address

if ($baseadd and $uscita eq 'doc' and $marcatura eq 'htmlnir')
	{
	if ($ris !~ m!<head>!i) 
		{$ris = "<head><base href=\"$baseadd\"></head>\n$ris";}
	else 	{$ris =~ s!<head>!<head>\n<base href=\"$baseadd\">!i;}
	}

# --- gestione output: browser o disco

if ($destinazione eq 'disco')
	{
	print "Content-Type: text/nir;\n";
	print "Content-Disposition: attachment; filename=\"$name.$ext\"\n\n";
	print $ris;
	}
else	{print $cgi->header;}

# --- conversione \n in <br> per input 'txt' e destinazione 'browser'

if ($destinazione eq 'browser'){ 

	if ($uscita eq 'rif'){ 
		$ris =~ s/^<pre>//;
		my ($urn, $start, $end, $ante, $rif, $post);
		my $i = 1;
		$ris = '<table align=center width="90%" border=1><tr bgColor=#bcdcd2>';
		$ris .= '<th colspan=2>Elenco dei riferimenti con Urn e contesti</th></tr>';
		foreach (@line){
#			($urn, $start, $end, $ante, $rif, $post) = split /\t/;
			($urn, $rif) = split /\t/;
			$ris .= "<tr bgColor=#d5f0e1><td rowspan=2 align=center>$i</td>";
#			$ris .= "<td>... $ante<br><b>$rif</b><br>$post ...</td></tr>\n";
			$ris .= "<td>$rif</td></tr>\n";
			$ris .= "<tr bgColor=#bcdcd2><td>$urn</td></tr>\n";
#			$ris .= "<td bgColor=#bcdcd2>$urn</td></tr>\n";
			$i++;
		}		
		$ris .= '</table>';
	} 
	elsif ($uscita eq 'list'){ 
		$ris =~ s/^<pre>//;
		my ($urn, $start, $end);
		$ris = '<table align=center bgColor=#d5f0e1 width="90%">';
		foreach (@line){
			($urn, $start, $end) = split /\t/;
			$ris .= "<tr><td>$urn</td><td align=right>$start</td><td align=right>$end</td></tr>\n";
		}		
		$ris .= '</table>';
	} 

	if ($formato eq 'txt'){ 
#		$ris =~ s/^<pre>//;
		($ris =~ s/(?<!-->)\n/<BR>/g) if $uscita eq 'doc';  # --- conversione \n in <br>
	} 

	if ($marcatura eq 'htmlnir'){ 			# --- apre nuova finestra dai link
		$ris =~ s!<a (?=href="http://www\.normattiva\.it/uri-res/N2Ls)!<a target=_nir !g;
	}

	if ($inputTipo eq 'tastiera') {
		open (FH, "$htmlFile");				# --- riscrive form + risultato
		while (<FH>) {
			s/\Q<!--OUT-->\E/$ris/ig;
			s/<\/TEXTAREA>/$text<\/TEXTAREA>/ig;
			print $_;
		}
		close (FH);
	}
	elsif ($formato eq 'txt')
	{
		print $cgi->start_html(-title=>'Risultato del Parser dei Riferimenti');
		print $ris;
		print $cgi->end_html;
	}
	else {print $ris;}
}

